<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="LabPlot" version="1">

<ToolBar name="mainToolBar" >
<text>Main</text>
<Action name="toggle_project_explorer_dock"/>
<Action name="toggle_properties_explorer_dock"/>
<Separator/>
<Action name="undo" />
<Action name="redo" />
<Separator/>
<Action name="new_workbook" />
<Action name="new_spreadsheet" />
<Action name="new_matrix" />
<Action name="new_worksheet" />
<Action name="new_datapicker" />
<Separator/>
<Action name="import" />
<Action name="export" />
</ToolBar>

<ToolBar name="worksheet_toolbar" >
<text>Worksheet</text>
</ToolBar>

<ToolBar name="datapicker_toolbar" >
<text>Datapicker</text>
</ToolBar>

<ToolBar name="spreadsheet_toolbar" >
<text>Spreadsheet</text>
</ToolBar>

<ToolBar name="cas_worksheet_toolbar" >
<text>CAS Worksheet</text>
</ToolBar>

<ToolBar name="cartesian_plot_toolbar" >
<text>CartesianPlot</text>
</ToolBar>

<MenuBar>
<Menu name="file" ><text>&amp;File</text>
<Menu name="new"><text>&amp;New</text>
	<Action name="new_workbook" />
	<Action name="new_spreadsheet" />
	<Action name="new_matrix" />
	<Action name="new_worksheet" />
	<Action name="new_notes" />
	<Action name="new_datapicker" />
	<Action name="new_script" />
	<Action name="new_folder" />
	<Separator/>
	<Action name="new_file_datasource" />
	<Action name="new_database_datasource" />
	<Separator/>
	<Menu name="new_cas_worksheet"><text>&amp;CAS Worksheet</text>
	    <ActionList name="backends_list" />
	</Menu>
</Menu>
<Separator/>
<Action name="import" />
<Action name="export" />
<Action name="project" />
</Menu>
<Menu name="edit"><text>&amp;Edit</text>
	<Action name="undo" />
	<Action name="redo" />
	<Action name="history" />
</Menu>
<Menu name="spreadsheet"><text>&amp;Spreadsheet</text></Menu>
<Menu name="matrix"><text>&amp;Matrix</text></Menu>
<Menu name="worksheet"><text>&amp;Worksheet</text></Menu>
<Menu name="cas_worksheet"><text>&amp;CAS Worksheet</text></Menu>
<Menu name="analysis"><text>&amp;Analysis</text></Menu>
<Menu name="datapicker"><text>&amp;Datapicker</text></Menu>
<!-- <Menu name="drawing"><text>&amp;Drawing</text></Menu> -->
<!-- <Menu name="script"><text>&amp;Script</text></Menu> -->
<Menu name="tools"><text>&amp;Tools</text>
            <Action name="edit_fits" />
</Menu>
<Menu name="windows"><text>&amp;Windows</text>
<Action name="close window" />
<Action name="close all windows" />
<Separator/>
<Action name="tile windows" />
<Action name="cascade windows" />
<Separator/>
<Action name="next window" />
<Action name="previous window" />
<Separator/>
<Action name="toggle_project_explorer_dock" />
<Action name="toggle_properties_explorer_dock" />
</Menu>
</MenuBar>
</gui>
