/***************************************************************************
    File                 : ScriptingEngineManager.h
    Project              : SciDAVis
    --------------------------------------------------------------------
    Copyright            : (C) 2008 Knut Franke
    Email (use @ for *)  : Knut.Franke*gmx.net
    Description          : Entry point for dealing with scripting.

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 *  This program is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *  GNU General Public License for more details.                           *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor,                    *
 *   Boston, MA  02110-1301  USA                                           *
 *                                                                         *
 ***************************************************************************/

#ifndef SCRIPTING_ENGINE_MANAGER_H
#define SCRIPTING_ENGINE_MANAGER_H

#include <QList>

class AbstractScriptingEngine;
class QStringList;

//! Entry point for dealing with scripting.
class ScriptingEngineManager
{
	private:
		ScriptingEngineManager();
		ScriptingEngineManager(const ScriptingEngineManager&) {}

		QList<AbstractScriptingEngine*> m_engines;

	public:
		static ScriptingEngineManager * instance();
		~ScriptingEngineManager();
		QStringList engineNames() const;
		AbstractScriptingEngine * engine(const QString &name);
};

#endif // ifndef SCRIPTING_ENGINE_MANAGER_H
